import chalk from 'chalk';
export var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["debug"] = 0] = "debug";
    LogLevel[LogLevel["verbose"] = 1] = "verbose";
    LogLevel[LogLevel["info"] = 2] = "info";
    LogLevel[LogLevel["warn"] = 3] = "warn";
    LogLevel[LogLevel["error"] = 4] = "error";
})(LogLevel || (LogLevel = {}));
export const BasicLoggerStyler = {
    getColor: (level, _nesting = 0) => {
        switch (level) {
            case LogLevel.error:
                return chalk.red;
            case LogLevel.warn:
                return chalk.yellow;
            case LogLevel.debug:
            case LogLevel.verbose:
                return chalk.white.dim;
            default:
                return chalk.white;
        }
    },
    formatPrefix: (prefix, level, _nesting = 0) => level === LogLevel.debug || level === LogLevel.verbose
        ? chalk.dim(prefix)
        : prefix,
    showPrefix: (_level) => {
        // Show prefix for all log levels
        return true;
    },
};
const loggerStyle = BasicLoggerStyler;
export class Logger {
    prefix;
    logLevel;
    logLevelIndex;
    parent;
    name;
    nesting;
    customPrefix;
    constructor({ name, parent = undefined, logLevel = parent?.logLevel ?? LogLevel.info, customPrefix, }) {
        this.customPrefix = customPrefix;
        this.name = name;
        this.parent = parent;
        this.logLevel = logLevel;
        this.logLevelIndex = logLevel;
        // Calculate indent level and offset based on parent chain
        this.nesting = 0;
        let offsetSpaces = 0;
        let currentParent = parent;
        while (currentParent) {
            offsetSpaces += 2;
            this.nesting++;
            currentParent = currentParent.parent;
        }
        this.prefix = ' '.repeat(offsetSpaces);
    }
    toStrings(messages) {
        return messages
            .map((message) => typeof message === 'object'
            ? JSON.stringify(message, null, 2)
            : String(message))
            .join(' ');
    }
    formatMessages(level, messages) {
        const formatted = this.toStrings(messages);
        const messageColor = loggerStyle.getColor(level, this.nesting);
        let combinedPrefix = this.prefix;
        if (loggerStyle.showPrefix(level)) {
            const prefix = loggerStyle.formatPrefix(`[${this.name}]`, level, this.nesting);
            if (this.customPrefix) {
                combinedPrefix = `${this.prefix}${this.customPrefix} `;
            }
            else {
                combinedPrefix = `${this.prefix}${prefix} `;
            }
        }
        return formatted
            .split('\n')
            .map((line) => `${combinedPrefix}${messageColor(line)}`)
            .join('\n');
    }
    log(level, ...messages) {
        if (level < this.logLevelIndex)
            return;
        console.log(this.formatMessages(level, messages));
    }
    debug(...messages) {
        if (LogLevel.debug < this.logLevelIndex)
            return;
        console.log(this.formatMessages(LogLevel.debug, messages));
    }
    verbose(...messages) {
        if (LogLevel.verbose < this.logLevelIndex)
            return;
        console.log(this.formatMessages(LogLevel.verbose, messages));
    }
    info(...messages) {
        if (LogLevel.info < this.logLevelIndex)
            return;
        console.log(this.formatMessages(LogLevel.info, messages));
    }
    warn(...messages) {
        if (LogLevel.warn < this.logLevelIndex)
            return;
        console.warn(this.formatMessages(LogLevel.warn, messages));
    }
    error(...messages) {
        console.error(this.formatMessages(LogLevel.error, messages));
    }
}
//# sourceMappingURL=logger.js.map